@@ -54,7 +54,7 @@
 #include <string.h>
 
 #include <glib.h>
-#include <glib/gi18n-lib.h> /* Declares _() - alias for gettext() */
+#include <glib/gi18n-lib.h> /* Declares _() alias for gettext() */
 
 #include <gtk/gtk.h>
 
@@ -72,7 +72,7 @@
 
 
 
-// #define DEBUG
+#define DEBUG
 
 #ifdef DEBUG
 
@@ -80,16 +80,12 @@
 // TODO: find out how to get debug logs from hildon-desktop
 #define g_debug g_message
 
-#define USER_AGENT_VERSION_STRING "(debug version)"
-
 #else
 
 // check g_debug statements for syntax errors but don't emit code
 // (compiler should optimize it away)
 #define g_debug(fmt...) do { if (0) { g_message(fmt); } } while (0)
 
-#define USER_AGENT_VERSION_STRING "(some version number)"
-
 #endif // DEBUG
 
 
@@ -124,11 +120,14 @@ stop_update_timer(CCCHHDoorIsDesktopWidget *self)
 
 // start update timer
 // returns TRUE if timer was started
-// returns FALSE if timer was already running
+// returns FALSE if timer was already running or auto_update disabled
 // TODO: check return code from g_timeout_add_seconds
 static inline gboolean
 start_update_timer(CCCHHDoorIsDesktopWidget *self)
 {
+	if (!self->auto_update)
+		return FALSE;
+
 	if (!self->timeout_handler) {
 		g_debug(WIDGET_NAME": starting update timer");
 		if (self->update_interval < UPDATE_INTERVAL_MIN) {
@@ -188,6 +187,7 @@ remove_connection_handler(CCCHHDoorIsDesktopWidget *self)
 
 
 
+// load settings from gconf
 static void
 load_settings(CCCHHDoorIsDesktopWidget *self)
 {
@@ -277,9 +277,27 @@ load_settings(CCCHHDoorIsDesktopWidget *self)
 		self->force_connection = boolval;
 		g_debug(WIDGET_NAME": load_settings: force_connection = %d", self->force_connection);
 	}
+
+
+	// auto_update: new since 0.1.10
+	// client_get_bool would return FALSE if key unset
+	GConfValue *confval = gconf_client_get_without_default(self->gconf_client,
+		GCONF_DIR"/auto_update", &error);
+
+	if (error) {
+		g_warning(WIDGET_NAME": failed to get configuration value auto_update: %s",
+			error->message);
+		g_error_free(error);
+		error = NULL;
+	} else if (confval) {
+		self->auto_update = gconf_value_get_bool(confval);
+		g_debug(WIDGET_NAME": load_settings: auto_update = %d", self->auto_update);
+		gconf_value_free(confval);
+	}
 }
 
 
+// save settings into gconf
 static void
 save_settings(CCCHHDoorIsDesktopWidget *self)
 {
@@ -333,14 +351,22 @@ save_settings(CCCHHDoorIsDesktopWidget *self)
 		g_error_free(error);
 		error = NULL;
 	}
+
+	if (!gconf_client_set_bool(self->gconf_client,
+		GCONF_DIR"/auto_update", self->auto_update, &error))
+	{
+		g_warning(WIDGET_NAME": failed to set configuration value auto_update: %s",
+			error->message);
+		g_error_free(error);
+		error = NULL;
+	}
 }
 
 
+// TODO: skip draw if not on active desktop
 static void
 draw (CCCHHDoorIsDesktopWidget *self, cairo_t *cr)
 {
-//  GtkWidget *widget = GTK_WIDGET(self);
-  
 	g_debug(WIDGET_NAME": draw()");
 
 	if (!self->image_current) {
@@ -369,12 +395,13 @@ draw (CCCHHDoorIsDesktopWidget *self, cairo_t *cr)
 		cairo_surface_destroy(image_overlay);
 	}
 
-	// if button pressed, hightligth with white mask
+	// if button pressed, hightlight with white mask
 	if (self->highlighted) {
 		cairo_set_source_rgba(cr, IMAGE_COLOR_HIGHLIGHT, IMAGE_ALPHA_HIGHLIGHT);
 		cairo_mask_surface(cr, self->image_current, 5, 5);
 	}
 
+// debug
 if (!self->timeout_handler) {
 	cairo_set_source_rgba(cr, 1, 0, 0, 0.47);
 	cairo_mask_surface(cr, self->image_current, 5, 5);
@@ -465,9 +492,6 @@ force_redraw (CCCHHDoorIsDesktopWidget *self)
 }
 
 
-// end private functions
-
-
 
 
 // async update with callbacks
@@ -605,6 +629,7 @@ fetch_update_async_read_cb (GnomeVFSAsyncHandle *handle, GnomeVFSResult result,
 
 	g_debug(WIDGET_NAME": read %"GNOME_VFS_SIZE_FORMAT_STR" bytes", bytes_read);
 
+// calling fetch_unlock in cancel_cb() would collide with process()
 //	self->gvfs_timeout_handler = g_timeout_add_seconds (TIMEOUT_CLOSE, fetch_update_cancel_cb, self);
 
 	// close file
@@ -687,7 +712,7 @@ fetch_update_async_close_cb (GnomeVFSAsyncHandle *handle, GnomeVFSResult result,
 
 	if (result != GNOME_VFS_OK) {
 		g_debug(WIDGET_NAME": error on close: %s", gnome_vfs_result_to_string (result));
-		// TODO: what now?
+		// Nothing else to do here
 	}
 }
 
@@ -706,6 +731,7 @@ fetch_update_process (CCCHHDoorIsDesktopWidget *self, dooris_state_t new_state,
 	if (new_state == self->last_state) {
 		g_debug(WIDGET_NAME": state unchanged");
 
+		// button sets force_banner, display current state even if unchanged
 		if (self->force_banner) {
 			self->force_banner = FALSE;
 			show_status_banner(self);
@@ -780,7 +806,6 @@ ccchh_dooris_desktop_widget_on_timeout (gpointer data)
 		return FALSE;
 	}
 
-
 	// stop timer when offline
 	// connection event starts it again
 	if (!self->connected) {
@@ -807,8 +832,9 @@ ccchh_dooris_desktop_widget_expose_event (GtkWidget *widget, GdkEventExpose *eve
 	if (!self->image_current) {
 		g_debug(WIDGET_NAME": expose event without image");
 		self->image_current = cairo_image_surface_create_from_png(IMAGE_DOORIS_FNORD);
-		// no state yet, begin update
-		fetch_update_start(self);
+		// no state yet, begin update if enabled
+		if (self->auto_update)
+			fetch_update_start(self);
 	}
 
 	cairo_t *cr  = gdk_cairo_create (widget->window);
@@ -852,6 +878,7 @@ ccchh_dooris_desktop_widget_on_button_press_event (GtkWidget *widget,
 
 
 // handle the leave notify event, remove highlight
+// TODO: should return immediately if not locked, i.e. no previous button-press
 static gboolean 
 ccchh_dooris_desktop_widget_on_button_leave_event (GtkWidget *widget, 
   GdkEventButton * event G_GNUC_UNUSED, gpointer user_data G_GNUC_UNUSED)
@@ -859,7 +886,7 @@ ccchh_dooris_desktop_widget_on_button_leave_event (GtkWidget *widget,
 	CCCHHDoorIsDesktopWidget *self = CCCHH_DOORIS_DESKTOP_WIDGET (widget);
 
 	g_debug(WIDGET_NAME": ccchh_dooris_desktop_widget_on_button_leave_event()");
- 
+
 	self->highlighted = FALSE;
 	force_redraw(self);
 
@@ -934,15 +961,14 @@ ccchh_dooris_desktop_widget_connection_event(ConIcConnection *connection,
 	switch (status) {
 		case CON_IC_STATUS_DISCONNECTING:
 
-			// save disconnecting IAP ID
-
-			if (self->disconnecting) {
-				g_free(self->disconnecting);
-				self->disconnecting = NULL;
+			if (self->disconnecting_iap) {
+				g_free(self->disconnecting_iap);
+				self->disconnecting_iap = NULL;
 			}
 
+			// save disconnecting IAP ID
 			if (iap_id)
-				self->disconnecting = g_strdup(iap_id);
+				self->disconnecting_iap = g_strdup(iap_id);
 
 			self->connected = FALSE;
 
@@ -967,8 +993,8 @@ ccchh_dooris_desktop_widget_connection_event(ConIcConnection *connection,
 
 		case CON_IC_STATUS_DISCONNECTED:
 
-			if (iap_id && self->disconnecting && self->connected &&
-					!strcmp(iap_id, self->disconnecting))
+			if (iap_id && self->disconnecting_iap && self->connected &&
+					!strcmp(iap_id, self->disconnecting_iap))
 			{
 
 				g_debug(WIDGET_NAME": ignoring offline event from disconnecting IAP");
@@ -988,9 +1014,9 @@ ccchh_dooris_desktop_widget_connection_event(ConIcConnection *connection,
 				force_redraw(self);
 			}
 
-			if (self->disconnecting) {
-				g_free(self->disconnecting);
-				self->disconnecting = NULL;
+			if (self->disconnecting_iap) {
+				g_free(self->disconnecting_iap);
+				self->disconnecting_iap = NULL;
 			}
 
 			break;
@@ -1031,12 +1057,23 @@ ccchh_dooris_desktop_widget_on_settings_dialog_response (GtkDialog *dialog,
 
 		// update interval
 		const gchar *choice = hildon_button_get_value(HILDON_BUTTON (self->update_picker));
-		self->update_interval = CLAMP(g_ascii_strtod (choice, NULL), UPDATE_INTERVAL_MIN, UPDATE_INTERVAL_MAX);
-		g_debug(WIDGET_NAME": new update interval: %ld\n", (long)self->update_interval);
+		if (strcmp(choice, _("disabled"))) {
+			// automatic updates enabled
+			self->auto_update = TRUE;
+			self->update_interval = CLAMP(g_ascii_strtod (choice, NULL), UPDATE_INTERVAL_MIN, UPDATE_INTERVAL_MAX);
+			g_debug(WIDGET_NAME": new update interval: %ld\n", (long)self->update_interval);
 
-		// if a timer was running, reset it
-		if (stop_update_timer(self)) {
+			// reset timer
+			stop_update_timer(self);
 			start_update_timer(self);
+		} else {
+			// automatic updates disabled
+			self->auto_update = FALSE;
+
+			g_debug(WIDGET_NAME": automatic updates disabled");
+
+			// if a timer was running, stop it
+			stop_update_timer(self);
 		}
 
 		// banner notifications
@@ -1055,14 +1092,15 @@ ccchh_dooris_desktop_widget_on_settings_dialog_response (GtkDialog *dialog,
 		} else {
 			remove_connection_handler(self);
 
-			start_update_timer(self);
-
 			// force connection state to online
 			// redraw if necessary to remove disconnection overlay
 			if (!self->connected) {
 				self->connected = TRUE;
 				force_redraw(self);
 			}
+
+			// start timer (if auto_update enabled, too)
+			start_update_timer(self);
 		}
 
 		// button forces internet connection
@@ -1074,6 +1112,8 @@ ccchh_dooris_desktop_widget_on_settings_dialog_response (GtkDialog *dialog,
 		}
 
 		save_settings(self);
+	} else {
+		g_debug(WIDGET_NAME": not saving new settings, response ID was %i", response_id);
 	}
 
 	gtk_widget_destroy (self->settings_window);
@@ -1093,11 +1133,24 @@ ccchh_dooris_desktop_widget_on_settings_dialog_response (GtkDialog *dialog,
 void update_picker_changed_value_cb (HildonPickerButton *picker, gpointer data)
 {
 	gdouble number = 0;
+
+	g_debug(WIDGET_NAME": update_picker_changed_value_cb()");
+
 	const gchar *choice = hildon_button_get_value(HILDON_BUTTON (picker));
-	number = CLAMP(g_ascii_strtod (choice, NULL), UPDATE_INTERVAL_MIN, UPDATE_INTERVAL_MAX);
-	gchar *value_text = g_strdup_printf ("%d", (int) number);
-	hildon_button_set_value(HILDON_BUTTON (picker), value_text);
-	g_free(value_text);
+	if (strcmp(choice, _("disabled"))) {
+		number = CLAMP(g_ascii_strtod (choice, NULL), UPDATE_INTERVAL_MIN, UPDATE_INTERVAL_MAX);
+		gchar *value_text = g_strdup_printf ("%d", (int) number);
+		hildon_button_set_value(HILDON_BUTTON (picker), value_text);
+		g_free(value_text);
+	}
+}
+
+// use_connection button callback - set force_connection (in-)sensitive
+void connection_button_toggled_cb (HildonCheckButton *button, gpointer data)
+{
+	CCCHHDoorIsDesktopWidget *self = CCCHH_DOORIS_DESKTOP_WIDGET(data);
+	g_debug(WIDGET_NAME": connection_button_toggled_cb()");
+	gtk_widget_set_sensitive(self->connforce_button, hildon_check_button_get_active(button));
 }
 
 
@@ -1146,42 +1199,57 @@ ccchh_dooris_desktop_widget_on_show_settings (GtkWidget *widget G_GNUC_UNUSED,
 		gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);
 
 		gint selidx = 0;
+		gint idx = 1;
 
 		// predefined choices for update interval selector
 		HildonTouchSelector *selector = HILDON_TOUCH_SELECTOR (hildon_touch_selector_entry_new_text());
+		hildon_touch_selector_append_text (selector, _("disabled"));
+		if (!selidx && !self->auto_update)
+			selidx = idx;
+		idx++;
 		hildon_touch_selector_append_text (selector, "5");
 		if (!selidx && self->update_interval <= 5)
-			selidx = 1;
+			selidx = idx;
+		idx++;
 		hildon_touch_selector_append_text (selector, "10");
 		if (!selidx && self->update_interval <= 10)
-			selidx = 2;
+			selidx = idx;
+		idx++;
 		hildon_touch_selector_append_text (selector, "15");
 		if (!selidx && self->update_interval <= 15)
-			selidx = 3;
+			selidx = idx;
+		idx++;
 		hildon_touch_selector_append_text (selector, "20");
 		if (!selidx && self->update_interval <= 20)
-			selidx = 4;
+			selidx = idx;
+		idx++;
 		hildon_touch_selector_append_text (selector, "25");
 		if (!selidx && self->update_interval <= 25)
-			selidx = 5;
+			selidx = idx;
+		idx++;
 		hildon_touch_selector_append_text (selector, "30");
 		if (!selidx && self->update_interval <= 30)
-			selidx = 6;
+			selidx = idx;
+		idx++;
 		hildon_touch_selector_append_text (selector, "45");
 		if (!selidx && self->update_interval <= 45)
-			selidx = 7;
+			selidx = idx;
+		idx++;
 		hildon_touch_selector_append_text (selector, "60");
 		if (!selidx && self->update_interval <= 60)
-			selidx = 8;
+			selidx = idx;
+		idx++;
 		hildon_touch_selector_set_active (selector, 0, selidx-1);
 
 		// update interval
 		self->update_picker = hildon_picker_button_new (HILDON_SIZE_FINGER_HEIGHT,
 			HILDON_BUTTON_ARRANGEMENT_VERTICAL);
 		hildon_picker_button_set_selector (HILDON_PICKER_BUTTON (self->update_picker), selector);
+
 		// install callback function to validate input
 		g_signal_connect (G_OBJECT (self->update_picker), "value-changed",
 			G_CALLBACK (update_picker_changed_value_cb), NULL);
+
 		// align left, center vertically
 		gtk_button_set_alignment(GTK_BUTTON(self->update_picker), 0, 0.5);
 		caption = hildon_caption_new(sizegroup, _("Update interval (minutes):"),
@@ -1207,6 +1275,9 @@ ccchh_dooris_desktop_widget_on_show_settings (GtkWidget *widget G_GNUC_UNUSED,
 		gtk_button_set_alignment(GTK_BUTTON(self->connection_button), 0, 0.5);
 		gtk_box_pack_start (GTK_BOX (hbox), self->connection_button, TRUE, TRUE, 0);
 
+		// install signal handler to toggle sensitivity of connforce_button
+		g_signal_connect (self->connection_button, "toggled", G_CALLBACK(connection_button_toggled_cb), self);
+
 		hbox = gtk_hbox_new (FALSE, HILDON_MARGIN_DEFAULT);
 		gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);
 
@@ -1222,13 +1293,19 @@ ccchh_dooris_desktop_widget_on_show_settings (GtkWidget *widget G_GNUC_UNUSED,
 	// set current values
 	gtk_entry_set_text(GTK_ENTRY(self->url_entry), self->update_url);
 
-	char *value_text = g_strdup_printf ("%d", self->update_interval);
+	gchar *value_text;
+	if (self->auto_update) {
+		value_text = g_strdup_printf ("%d", self->update_interval);
+	} else {
+		value_text = g_strdup(_("disabled"));
+	}
 	hildon_button_set_value(HILDON_BUTTON (self->update_picker), value_text);
 	g_free(value_text);
 
 	hildon_check_button_set_active(HILDON_CHECK_BUTTON(self->notify_button), self->notify_enabled);
 	hildon_check_button_set_active(HILDON_CHECK_BUTTON(self->connection_button), self->use_connection);
 	hildon_check_button_set_active(HILDON_CHECK_BUTTON(self->connforce_button), self->force_connection);
+	gtk_widget_set_sensitive(self->connforce_button, self->use_connection);
 
 	gtk_widget_show_all (self->settings_window);
 }
@@ -1319,9 +1396,9 @@ ccchh_dooris_desktop_widget_finalize (GObject *object)
 		self->gvfs_input_buffer = NULL;
 	}
 
-	if (self->disconnecting) {
-		g_free(self->disconnecting);
-		self->disconnecting = NULL;
+	if (self->disconnecting_iap) {
+		g_free(self->disconnecting_iap);
+		self->disconnecting_iap = NULL;
 	}
 
 	if (self->gconf_client) {
@@ -1381,6 +1458,7 @@ ccchh_dooris_desktop_widget_init (CCCHHDoorIsDesktopWidget *self)
 //	self->update_url = gnome_vfs_make_uri_from_input("http://formularfetischisten.de/~packbart/temp/door.txt");
 	self->use_connection = TRUE;
 	self->force_connection = TRUE;
+	self->auto_update = TRUE;
 
 	// initialize state
 	// this combination ensures that update() sets an image on first call
@@ -1403,7 +1481,7 @@ ccchh_dooris_desktop_widget_init (CCCHHDoorIsDesktopWidget *self)
 
 	self->force_banner = FALSE;
 
-	self->disconnecting = NULL;
+	self->disconnecting_iap = NULL;
 
 	self->gvfs_handle = NULL;
 	self->gvfs_input_buffer = NULL;
@@ -1421,7 +1499,7 @@ ccchh_dooris_desktop_widget_init (CCCHHDoorIsDesktopWidget *self)
 		GDK_BUTTON_RELEASE_MASK | GDK_LEAVE_NOTIFY_MASK;
 	gtk_widget_set_events (GTK_WIDGET (self), mask);
      
-	/* Handle a mouse button press: */
+	/* Handle mouse button events: */
 	g_signal_connect (self, "button-press-event", 
 		G_CALLBACK (ccchh_dooris_desktop_widget_on_button_press_event), NULL);
 
@@ -1454,6 +1532,7 @@ ccchh_dooris_desktop_widget_init (CCCHHDoorIsDesktopWidget *self)
 		self->connected = TRUE;
 		start_update_timer(self);
 	}
+
 	g_debug(WIDGET_NAME": initial URL: %s\n", self->update_url);
 }
 
